namespace SilverlightOpenAccessIntegration.Web
{
    using System;
    using System.Collections;
    using System.Data.Services;
    using System.Linq;
    using System.Reflection;
    using Telerik.OpenAccess;

    [IgnoreProperties("SummaryOfSalesByYears",
        "SummaryOfSalesByQuarters",
        "SalesTotalsByAmounts",
        "SalesByCategories",
        "QuarterlyOrders",
        "ProductsByCategories",
        "ProductsAboveAveragePrices",
        "ProductSalesFor1997s",
        "OrdersQries",
        "OrderSubtotals",
        "OrderDetailsExtendeds",
        "Invoices",
        "CustomerAndSuppliersByCities",
        "CurrentProductLists",
        "CategorySalesFor1997s",
        "AlphabeticalListOfProducts")]
    public partial class DataManager : IDisposable, ICloneable, IUpdatable
    {
        #region Private Memebrs

        /// <summary>
        /// OpenAccess Context
        /// </summary>
        private OpenAccessContext _openAccessConext = null;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="OADataContext"/> class.
        /// </summary>
        public DataManager()
        {
            this._openAccessConext = new NorthwindLibrary.NorthwindOAEntityDiagrams();
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="OADataContext"/> class.
        /// </summary>
        /// <param name="scope">Object scope <see cref="IObjectScope"/>.</param>
        public DataManager(OpenAccessContext context)
        {
            this._openAccessConext = context;
        }

        #endregion

        #region Private helper Functions

        /// <summary>
        /// Roll Back the trancation 
        /// </summary>
        /// <param name="scope"> The scope of the trancation </param>
        public void HandleException(Exception ex)
        {
            try
            {
                if (this._openAccessConext != null)
                {
                    this._openAccessConext.ClearChanges();
                }
            }
            finally
            {
                throw ex;
            }
        }

        #endregion

        #region Public Properties

        /// <summary>
        /// Gets the Categories.
        /// </summary>
        /// <value>The Categories.</value>
        public IQueryable<NorthwindLibrary.Category> Categories
        {
            get
            {
                return this._openAccessConext.GetAll<NorthwindLibrary.Category>();
            }
        }
        /// <summary>
        /// Gets the CustomerDemographics.
        /// </summary>
        /// <value>The CustomerDemographics.</value>
        public IQueryable<NorthwindLibrary.CustomerDemographic> CustomerDemographics
        {
            get
            {
                return this._openAccessConext.GetAll<NorthwindLibrary.CustomerDemographic>();
            }
        }
        /// <summary>
        /// Gets the Customers.
        /// </summary>
        /// <value>The Customers.</value>
        public IQueryable<NorthwindLibrary.Customer> Customers
        {
            get
            {
                return this._openAccessConext.GetAll<NorthwindLibrary.Customer>();
            }
        }
        /// <summary>
        /// Gets the Employees.
        /// </summary>
        /// <value>The Employees.</value>
        public IQueryable<NorthwindLibrary.Employee> Employees
        {
            get
            {
                return this._openAccessConext.GetAll<NorthwindLibrary.Employee>();
            }
        }
        /// <summary>
        /// Gets the LargeEmployees.
        /// </summary>
        /// <value>The LargeEmployees.</value>
        public IQueryable<NorthwindLibrary.LargeEmployee> LargeEmployees
        {
            get
            {
                return this._openAccessConext.GetAll<NorthwindLibrary.LargeEmployee>();
            }
        }
        /// <summary>
        /// Gets the Messages.
        /// </summary>
        /// <value>The Messages.</value>
        public IQueryable<NorthwindLibrary.Message> Messages
        {
            get
            {
                return this._openAccessConext.GetAll<NorthwindLibrary.Message>();
            }
        }
        /// <summary>
        /// Gets the OrderDetails.
        /// </summary>
        /// <value>The OrderDetails.</value>
        public IQueryable<NorthwindLibrary.OrderDetail> OrderDetails
        {
            get
            {
                return this._openAccessConext.GetAll<NorthwindLibrary.OrderDetail>();
            }
        }
        /// <summary>
        /// Gets the Orders.
        /// </summary>
        /// <value>The Orders.</value>
        public IQueryable<NorthwindLibrary.Order> Orders
        {
            get
            {
                return this._openAccessConext.GetAll<NorthwindLibrary.Order>();
            }
        }
        /// <summary>
        /// Gets the Products.
        /// </summary>
        /// <value>The Products.</value>
        public IQueryable<NorthwindLibrary.Product> Products
        {
            get
            {
                return this._openAccessConext.GetAll<NorthwindLibrary.Product>();
            }
        }
        /// <summary>
        /// Gets the RadProducts.
        /// </summary>
        /// <value>The RadProducts.</value>
        public IQueryable<NorthwindLibrary.RadProduct> RadProducts
        {
            get
            {
                return this._openAccessConext.GetAll<NorthwindLibrary.RadProduct>();
            }
        }
        /// <summary>
        /// Gets the Regions.
        /// </summary>
        /// <value>The Regions.</value>
        public IQueryable<NorthwindLibrary.Region> Regions
        {
            get
            {
                return this._openAccessConext.GetAll<NorthwindLibrary.Region>();
            }
        }
        /// <summary>
        /// Gets the Shippers.
        /// </summary>
        /// <value>The Shippers.</value>
        public IQueryable<NorthwindLibrary.Shipper> Shippers
        {
            get
            {
                return this._openAccessConext.GetAll<NorthwindLibrary.Shipper>();
            }
        }
        /// <summary>
        /// Gets the Suppliers.
        /// </summary>
        /// <value>The Suppliers.</value>
        public IQueryable<NorthwindLibrary.Supplier> Suppliers
        {
            get
            {
                return this._openAccessConext.GetAll<NorthwindLibrary.Supplier>();
            }
        }
        /// <summary>
        /// Gets the Territories.
        /// </summary>
        /// <value>The Territories.</value>
        public IQueryable<NorthwindLibrary.Territory> Territories
        {
            get
            {
                return this._openAccessConext.GetAll<NorthwindLibrary.Territory>();
            }
        }
        /// <summary>
        /// Gets the Users.
        /// </summary>
        /// <value>The Users.</value>
        public IQueryable<NorthwindLibrary.User> Users
        {
            get
            {
                return this._openAccessConext.GetAll<NorthwindLibrary.User>();
            }
        }


        #region Views

        /// <summary>
        /// Gets the AlphabeticalListOfProducts.
        /// </summary>
        /// <value>The AlphabeticalListOfProducts.</value>
        public IQueryable<NorthwindLibrary.AlphabeticalListOfProduct> AlphabeticalListOfProducts
        {
            get
            {
                return this._openAccessConext.GetAll<NorthwindLibrary.AlphabeticalListOfProduct>();
            }
        }
        /// <summary>
        /// Gets the CategorySalesFor1997s.
        /// </summary>
        /// <value>The CategorySalesFor1997s.</value>
        public IQueryable<NorthwindLibrary.CategorySalesFor1997> CategorySalesFor1997s
        {
            get
            {
                return this._openAccessConext.GetAll<NorthwindLibrary.CategorySalesFor1997>();
            }
        }
        /// <summary>
        /// Gets the CurrentProductLists.
        /// </summary>
        /// <value>The CurrentProductLists.</value>
        public IQueryable<NorthwindLibrary.CurrentProductList> CurrentProductLists
        {
            get
            {
                return this._openAccessConext.GetAll<NorthwindLibrary.CurrentProductList>();
            }
        }
        /// <summary>
        /// Gets the CustomerAndSuppliersByCities.
        /// </summary>
        /// <value>The CustomerAndSuppliersByCities.</value>
        public IQueryable<NorthwindLibrary.CustomerAndSuppliersByCity> CustomerAndSuppliersByCities
        {
            get
            {
                return this._openAccessConext.GetAll<NorthwindLibrary.CustomerAndSuppliersByCity>();
            }
        }
        /// <summary>
        /// Gets the Invoices.
        /// </summary>
        /// <value>The Invoices.</value>
        public IQueryable<NorthwindLibrary.Invoice> Invoices
        {
            get
            {
                return this._openAccessConext.GetAll<NorthwindLibrary.Invoice>();
            }
        }
        /// <summary>
        /// Gets the OrderDetailsExtendeds.
        /// </summary>
        /// <value>The OrderDetailsExtendeds.</value>
        public IQueryable<NorthwindLibrary.OrderDetailsExtended> OrderDetailsExtendeds
        {
            get
            {
                return this._openAccessConext.GetAll<NorthwindLibrary.OrderDetailsExtended>();
            }
        }
        /// <summary>
        /// Gets the OrderSubtotals.
        /// </summary>
        /// <value>The OrderSubtotals.</value>
        public IQueryable<NorthwindLibrary.OrderSubtotal> OrderSubtotals
        {
            get
            {
                return this._openAccessConext.GetAll<NorthwindLibrary.OrderSubtotal>();
            }
        }
        /// <summary>
        /// Gets the OrdersQries.
        /// </summary>
        /// <value>The OrdersQries.</value>
        public IQueryable<NorthwindLibrary.OrdersQry> OrdersQries
        {
            get
            {
                return this._openAccessConext.GetAll<NorthwindLibrary.OrdersQry>();
            }
        }
        /// <summary>
        /// Gets the ProductSalesFor1997s.
        /// </summary>
        /// <value>The ProductSalesFor1997s.</value>
        public IQueryable<NorthwindLibrary.ProductSalesFor1997> ProductSalesFor1997s
        {
            get
            {
                return this._openAccessConext.GetAll<NorthwindLibrary.ProductSalesFor1997>();
            }
        }
        /// <summary>
        /// Gets the ProductsAboveAveragePrices.
        /// </summary>
        /// <value>The ProductsAboveAveragePrices.</value>
        public IQueryable<NorthwindLibrary.ProductsAboveAveragePrice> ProductsAboveAveragePrices
        {
            get
            {
                return this._openAccessConext.GetAll<NorthwindLibrary.ProductsAboveAveragePrice>();
            }
        }
        /// <summary>
        /// Gets the ProductsByCategories.
        /// </summary>
        /// <value>The ProductsByCategories.</value>
        public IQueryable<NorthwindLibrary.ProductsByCategory> ProductsByCategories
        {
            get
            {
                return this._openAccessConext.GetAll<NorthwindLibrary.ProductsByCategory>();
            }
        }
        /// <summary>
        /// Gets the QuarterlyOrders.
        /// </summary>
        /// <value>The QuarterlyOrders.</value>
        public IQueryable<NorthwindLibrary.QuarterlyOrder> QuarterlyOrders
        {
            get
            {
                return this._openAccessConext.GetAll<NorthwindLibrary.QuarterlyOrder>();
            }
        }
        /// <summary>
        /// Gets the SalesByCategories.
        /// </summary>
        /// <value>The SalesByCategories.</value>
        public IQueryable<NorthwindLibrary.SalesByCategory> SalesByCategories
        {
            get
            {
                return this._openAccessConext.GetAll<NorthwindLibrary.SalesByCategory>();
            }
        }
        /// <summary>
        /// Gets the SalesTotalsByAmounts.
        /// </summary>
        /// <value>The SalesTotalsByAmounts.</value>
        public IQueryable<NorthwindLibrary.SalesTotalsByAmount> SalesTotalsByAmounts
        {
            get
            {
                return this._openAccessConext.GetAll<NorthwindLibrary.SalesTotalsByAmount>();
            }
        }
        /// <summary>
        /// Gets the SummaryOfSalesByQuarters.
        /// </summary>
        /// <value>The SummaryOfSalesByQuarters.</value>
        public IQueryable<NorthwindLibrary.SummaryOfSalesByQuarter> SummaryOfSalesByQuarters
        {
            get
            {
                return this._openAccessConext.GetAll<NorthwindLibrary.SummaryOfSalesByQuarter>();
            }
        }
        /// <summary>
        /// Gets the SummaryOfSalesByYears.
        /// </summary>
        /// <value>The SummaryOfSalesByYears.</value>
        public IQueryable<NorthwindLibrary.SummaryOfSalesByYear> SummaryOfSalesByYears
        {
            get
            {
                return this._openAccessConext.GetAll<NorthwindLibrary.SummaryOfSalesByYear>();
            }
        }

        #endregion

        #endregion

        #region IUpdatable Members

        /// <summary>
        /// Creates the resource of the specified type and that belongs to the specified container.
        /// </summary>
        /// <param name="containerName">The name of the entity set to which the resource belongs.</param>
        /// <param name="fullTypeName">The full namespace-qualified type name of the resource.</param>
        /// <returns>
        /// The object representing a resource of specified type and belonging to the specified container.
        /// </returns>
        public object CreateResource(string containerName, string fullTypeName)
        {
            try
            {
                object resource = Activator.CreateInstance(_openAccessConext.GetType().Assembly.GetType(fullTypeName));
                this._openAccessConext.Add(resource);
                return resource;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

        /// <summary>
        /// Gets the resource of the specified type identified by a query and type name.
        /// </summary>
        /// <param name="query">Language integratee query(LINQ) pointing to a particular resource.</param>
        /// <param name="fullTypeName">The fully qualified type name of resource.</param>
        /// <returns>
        /// An opaque object representing a resource of the specified type, referenced by the specified query.
        /// </returns>
        public object GetResource(IQueryable query, string fullTypeName)
        {
            try
            {
                object resource = null;

                foreach (object o in query)
                {
                    if (resource != null)
                    {
                        throw new Exception("Expected a single response");
                    }
                    resource = o;
                }

                if (resource == null)
                    return new Telerik.OpenAccess.Exceptions.NoSuchObjectException("Object cannot be found.", null);

                return resource;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

        /// <summary>
        /// Updates the resource identified by the parameter <paramref name="resource"/>.
        /// </summary>
        /// <param name="resource">The resource to be updated.</param>
        /// <returns>resouce value object</returns>
        public object ResetResource(object resource)
        {
            try
            {

                this._openAccessConext.Refresh(
                    RefreshMode.OverwriteChangesFromStore, resource);
                return resource;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

        /// <summary>
        /// Sets the value of the property with the specified name on the target resource to the specified property value.
        /// </summary>
        /// <param name="targetResource">The target object that defines the property.</param>
        /// <param name="propertyName">The name of the property whose value needs to be updated.</param>
        /// <param name="propertyValue">The property value for update.</param>
        public void SetValue(object targetResource, string propertyName, object propertyValue)
        {
            try
            {
                PropertyInfo pi = targetResource.GetType().GetProperty(propertyName);
                if (pi == null)
                    throw new Exception("Can't find property");

                pi.SetValue(targetResource, propertyValue, null);
            }
            catch (OpenAccessException ex)
            {
                HandleException(ex);
            }
        }

        /// <summary>
        /// Gets the value of the specified property on the target object.
        /// </summary>
        /// <param name="targetResource">An opaque object that represents a resource.</param>
        /// <param name="propertyName">The name of the property whose value needs to be retrieved.</param>
        /// <returns>value of the resource</returns>
        public object GetValue(object targetResource, string propertyName)
        {
            try
            {
                PropertyInfo pi = targetResource.GetType().GetProperty(propertyName);
                if (pi == null)
                    throw new Exception("Can't find property");

                return pi.GetValue(targetResource, null);
            }
            catch
            {
                return null;
            }
        }

        /// <summary>
        /// Sets the value of the specified reference property on the target object.
        /// </summary>
        /// <param name="targetResource">The target object that defines the property.</param>
        /// <param name="propertyName">The name of the property whose value needs to be updated.</param>
        /// <param name="propertyValue">The property value to be updated.</param>
        public void SetReference(object targetResource, string propertyName, object propertyValue)
        {
            SetValue(targetResource, propertyName, propertyValue);
        }

        /// <summary>
        /// Adds the specified value to the collection.
        /// </summary>
        /// <param name="targetResource">Target object that defines the property.</param>
        /// <param name="propertyName">The name of the collection property to which the resource should be added..</param>
        /// <param name="resourceToBeAdded">The opaque object representing the resource to be added.</param>
        public void AddReferenceToCollection(object targetResource, string propertyName, object resourceToBeAdded)
        {
            PropertyInfo pi = targetResource.GetType().GetProperty(propertyName);
            if (pi == null)
                throw new Exception("Can't find property");
            IList collection = (IList)pi.GetValue(targetResource, null);

            collection.Add(resourceToBeAdded);
        }

        /// <summary>
        /// Removes the specified value from the collection.
        /// </summary>
        /// <param name="targetResource">The target object that defines the property.</param>
        /// <param name="propertyName">The name of the property whose value needs to be updated.</param>
        /// <param name="resourceToBeRemoved">The property value that needs to be removed.</param>
        public void RemoveReferenceFromCollection(object targetResource, string propertyName, object resourceToBeRemoved)
        {
            try
            {
                PropertyInfo pi = targetResource.GetType().GetProperty(propertyName);

                if (pi == null)
                    throw new Exception("Can't find property");
                IList collection = (IList)pi.GetValue(targetResource, null);

                collection.Remove(resourceToBeRemoved);
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
        }

        /// <summary>
        /// Deletes the specified resource.
        /// </summary>
        /// <param name="targetResource">The resource to be deleted.</param>
        public void DeleteResource(object targetResource)
        {
            try
            {
                this._openAccessConext.Delete(targetResource);
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
        }

        /// <summary>
        /// Saves all the changes that have been made by using the <see cref="T:System.Data.Services.IUpdatable"/> APIs.
        /// </summary>
        public void SaveChanges()
        {
            try
            {
                this._openAccessConext.SaveChanges();
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
        }

        /// <summary>
        /// Returns the instance of the resource represented by the specified resource object.
        /// </summary>
        /// <param name="resource">The object representing the resource whose instance needs to be retrieved.</param>
        /// <returns>
        /// Returns the instance of the resource represented by the specified resource object.
        /// </returns>
        public object ResolveResource(object resource)
        {
            try
            {
                return resource;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

        /// <summary>
        /// Cancels a change to the data.
        /// </summary>
        public void ClearChanges()
        {
            try
            {
                this._openAccessConext.ClearChanges();
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
        }

        #endregion

        #region ICloneable Members

        /// <summary>
        /// Creates a new object that is a copy of the current instance.
        /// </summary>
        /// <returns>
        /// A new object that is a copy of this instance.
        /// </returns>
        public virtual object Clone()
        {
            return Activator.CreateInstance(GetType(), this._openAccessConext);
        }

        #endregion

        #region IDisposable Members

        /// <summary>
        /// Performs application-defined tasks associated with freeing, releasing, or resetting unmanaged resources.
        /// </summary>
        public void Dispose()
        {
            if (this._openAccessConext != null)
            {
                this._openAccessConext.Dispose();
                this._openAccessConext = null;
            }
        }

        #endregion
    }
}
